Wammu
=====

GUI pro Gammu.

.. image:: https://hosted.weblate.org/widgets/gammu-status-badge.png
    :alt: Stav překladu :target:
    https://hosted.weblate.org/engage/gammu/?utm_source=widget

Domovská stránka
================

<http://cs.wammu.eu/>

Licence
=======

GNU GPL verze 2.

První spuštění
==============

Při prvním startu budete požádáni o nastavení připojení k telefonu. Pokud
jste ještě nepoužili Gammu/Wammu, bude vám navrhnuto vyhledání telefonu,
které by mělo ve většině případů nalézt správnou konfiguraci.

Použití
=======

Nejdříve se musíte připojit k telefonu, pak s ním můžete provádět nějaké
operace. Pro vytváření záznamů a pro import nepotřebujete načíst data z
telefonu, pro ostatní musíte (překvapivě? :-)).

Všechny akce s aktuálními daty jsou dostupné také z kontextového menu na
každé položky. Můžete také použít klávesu Enter pro upravení položky a
Delete pro vymazání.

Zálohování uloží všechny položky, který jste již načetli z telefonu.

Hlášení chyb
============

Nalezené chyby prosím nahlašte na <http://bugs.wammu.eu>.

Překládání
==========

Můžete pomoci přeložit Wammu do jazyka, který umíte -
<https://hosted.weblate.org/projects/wammu/>.

Správa revizí
=============

Vývoj probíhá v Gitu, hlavní vývojovou větev naleznete na
<git://github.com/gammu/wammu.git> a prohlížet si jí můžete na stránkách
<https://github.com/gammu/wammu>.
